//package digisec.security.x509;

import java.io.IOException;
import java.security.*;
import java.security.cert.*;
import java.util.Date;
import sun.security.pkcs.PKCS10;
import sun.security.x509.*;

// Referenced classes of package sun.security.x509:
//            AlgorithmId, CertificateAlgorithmId, CertificateIssuerName, CertificateSerialNumber, 
//            CertificateSubjectName, CertificateValidity, CertificateVersion, CertificateX509Key, 
//            X500Signer, X509Cert, X509CertImpl, X509CertInfo, 
//            X509Key, X500Name

public final class CertAndKeyGen
{

    public CertAndKeyGen(String s, String s1)
        throws NoSuchAlgorithmException, NoSuchProviderException
    {
        keyGen = KeyPairGenerator.getInstance(s);
        sigAlg = s1;
    }

    public CertAndKeyGen(String s, String s1, String provider)
        throws NoSuchAlgorithmException, NoSuchProviderException
    {
        keyGen = KeyPairGenerator.getInstance(s, provider);
        sigAlg = s1;
    }

    public void generate(int i)
        throws InvalidKeyException
    {
        KeyPair keypair;
        try
        {
            if(prng == null)
                prng = new SecureRandom();
            keyGen.initialize(i, prng);
            keypair = keyGen.generateKeyPair();
        }
        catch(Exception exception)
        {
            throw new IllegalArgumentException(exception.getMessage());
        }
        publicKey = keypair.getPublic();
        privateKey = keypair.getPrivate();
    }

    public PKCS10 getCertRequest(X500Name x500name)
        throws InvalidKeyException, SignatureException
    {
        PKCS10 pkcs10 = new PKCS10(publicKey);
        try
        {
            pkcs10.encodeAndSign(getSigner(x500name));
        }
        catch(CertificateException _ex)
        {
            throw new SignatureException(sigAlg + " CertificateException");
        }
        catch(IOException _ex)
        {
            throw new SignatureException(sigAlg + " IOException");
        }
        catch(NoSuchAlgorithmException _ex)
        {
            throw new SignatureException(sigAlg + " unavailable?");
        }
        return pkcs10;
    }

    public PrivateKey getPrivateKey()
    {
        return privateKey;
    }

    public X509Key getPublicKey()
    {
        if(!(publicKey instanceof X509Key))
            return null;
        else
            return (X509Key)publicKey;
    }

    public X509Cert getSelfCert(X500Name x500name, long l)
        throws InvalidKeyException, SignatureException, NoSuchAlgorithmException
    {
        try
        {
            X509Certificate x509certificate = getSelfCertificate(x500name, l);
            return new X509Cert(x509certificate.getEncoded());
        }
        catch(CertificateException certificateexception)
        {
            throw new SignatureException(certificateexception.getMessage());
        }
        catch(NoSuchProviderException nosuchproviderexception)
        {
            throw new NoSuchAlgorithmException(nosuchproviderexception.getMessage());
        }
        catch(IOException ioexception)
        {
            throw new SignatureException(ioexception.getMessage());
        }
    }

    public X509Certificate getSelfCertificate(X500Name x500name, long l)
        throws CertificateException, InvalidKeyException, SignatureException, NoSuchAlgorithmException, NoSuchProviderException
    {
        try
        {
            X500Signer x500signer = getSigner(x500name);
            Date date = new Date();
            Date date1 = new Date();
            date1.setTime(date1.getTime() + l * 1000L);
            CertificateValidity certificatevalidity = new CertificateValidity(date, date1);
            X509CertInfo x509certinfo = new X509CertInfo();
            x509certinfo.set("version", new CertificateVersion(0));
            x509certinfo.set("serialNumber", new CertificateSerialNumber((int)(date.getTime() / 1000L)));
            AlgorithmId algorithmid = x500signer.getAlgorithmId();
            x509certinfo.set("algorithmID", new CertificateAlgorithmId(algorithmid));
            x509certinfo.set("subject", new CertificateSubjectName(x500name));
            x509certinfo.set("key", new CertificateX509Key(publicKey));
            x509certinfo.set("validity", certificatevalidity);
            x509certinfo.set("issuer", new CertificateIssuerName(x500signer.getSigner()));
            X509CertImpl x509certimpl = new X509CertImpl(x509certinfo);
            x509certimpl.sign(privateKey, algorithmid.getName());
            return x509certimpl;
        }
        catch(IOException ioexception)
        {
            throw new CertificateEncodingException("getSelfCert: " + ioexception.getMessage());
        }
    }

    private X500Signer getSigner(X500Name x500name)
        throws InvalidKeyException, NoSuchAlgorithmException
    {
        Signature signature = Signature.getInstance(sigAlg);
        signature.initSign(privateKey);
        return new X500Signer(signature, x500name);
    }

    public void setRandom(SecureRandom securerandom)
    {
        prng = securerandom;
    }

    private SecureRandom prng;
    private String sigAlg;
    private KeyPairGenerator keyGen;
    private PublicKey publicKey;
    private PrivateKey privateKey;
}